@@ -1,11 +1,12 @@
 from __future__ import annotations
 
 import time
-from typing import Union, Generator, Optional, Tuple, Callable, Set
+from typing import Union, Generator, Optional, Tuple, Callable, Set, Iterable
 import traceback
 import numpy
 import itertools
 import warnings
+import copy
 
 from amulet import log
 from amulet.api.block import Block, UniversalAirBlock
@@ -14,7 +15,7 @@ from amulet.api.entity import Entity
 from amulet.api.registry import BlockManager
 from amulet.api.registry.biome_manager import BiomeManager
 from amulet.api.errors import ChunkDoesNotExist, ChunkLoadError, DimensionDoesNotExist
-from amulet.api.chunk import Chunk
+from amulet.api.chunk import Chunk, EntityList
 from amulet.api.selection import SelectionGroup, SelectionBox
 from amulet.api.data_types import (
     Dimension,
@@ -23,6 +24,7 @@ from amulet.api.data_types import (
     FloatTriplet,
     ChunkCoordinates,
 )
+from amulet.api.chunk.status import StatusFormats
 from amulet.utils.generator import generator_unpacker
 from amulet.utils.world_utils import block_coords_to_chunk_coords
 from .chunk_manager import ChunkManager
@@ -264,7 +266,7 @@ class BaseLevel:
     def get_moved_coord_slice_box(
         self,
         dimension: Dimension,
-        destination_origin: Tuple[int, int, int],
+        destination_origin: BlockCoordinates,
         selection: Optional[Union[SelectionGroup, SelectionBox]] = None,
         destination_sub_chunk_shape: Optional[int] = None,
         yield_missing_chunks: bool = False,
@@ -336,7 +338,7 @@ class BaseLevel:
     def get_moved_chunk_slice_box(
         self,
         dimension: Dimension,
-        destination_origin: Tuple[int, int, int],
+        destination_origin: BlockCoordinates,
         selection: Optional[Union[SelectionGroup, SelectionBox]] = None,
         destination_sub_chunk_shape: Optional[int] = None,
         create_missing_chunks: bool = False,
@@ -458,7 +460,8 @@ class BaseLevel:
                         log.info(f"Converting chunk {dimension} {cx}, {cz}")
                         try:
                             chunk = self.level_wrapper.load_chunk(cx, cz, dimension)
-                            wrapper.commit_chunk(chunk, dimension)
+                            if chunk.status.as_type(StatusFormats.Java_14) == "full":
+                                wrapper.commit_chunk(chunk, dimension)
                         except ChunkLoadError:
                             log.info(f"Error loading chunk {cx} {cz}", exc_info=True)
                         chunk_index += 1
@@ -811,6 +814,39 @@ class BaseLevel:
             del chunk.block_entities[(x, y, z)]
         chunk.changed = True
 
+    def get_native_entities(
+        self, cx: int, cz: int, dimension: Dimension
+    ) -> Tuple[EntityList, VersionIdentifierType]:
+        """
+        Get a list of entities in the native format from a given chunk.
+        This currently returns the raw data from the chunk but in the future will convert to the world version format.
+
+        :param cx: The chunk x position
+        :param cz: The chunk z position
+        :param dimension: The dimension of the chunk.
+        :return: A copy of the list of entities and the version format they are in.
+        """
+        chunk = self.get_chunk(cx, cz, dimension)
+        # To make this forwards compatible this needs to be deep copied
+        return copy.deepcopy(chunk._native_entities), chunk._native_version
+
+    def set_native_entites(
+        self, cx: int, cz: int, dimension: Dimension, entities: Iterable[Entity]
+    ):
+        """
+        Set the entities in the native format.
+        Note that the format must be compatible with `level_wrapper.max_world_version`.
+
+        :param cx: The chunk x position
+        :param cz: The chunk z position
+        :param dimension: The dimension of the chunk.
+        :param entities: The entities to set on the chunk.
+        """
+        chunk = self.get_chunk(cx, cz, dimension)
+        chunk._native_entities = EntityList(copy.deepcopy(entities))
+        chunk._native_version = self.level_wrapper.max_world_version
+        chunk.changed = True
+
     # def get_entities_in_box(
     #     self, box: "SelectionGroup"
     # ) -> Generator[Tuple[Coordinates, List[object]], None, None]:
