@@ -1,3 +1,4 @@
+import sys
 from typing import TYPE_CHECKING, Tuple, Generator, Optional
 import numpy
 
@@ -106,6 +107,7 @@ def clone(
 
         # TODO: I don't know if this is feasible for large boxes: get the intersection of the source and destination selections and iterate over that to minimise work
         if any(rotation) or any(s != 1 for s in scale):
+            # if the selection needs transforming
             rotation_radians = tuple(numpy.radians(rotation))
             transform = numpy.matmul(
                 transform_matrix(scale, rotation_radians, location),
@@ -197,10 +199,14 @@ def clone(
                                                 mask = block_ids == block_id
                                                 dst_blocks_ = dst_blocks[mask]
 
+                                                transformed_block = src_structure.translation_manager.transform_universal_block(
+                                                    block, transform
+                                                )
+
                                                 dst_chunk.blocks.get_sub_chunk(dst_cy)[
                                                     tuple(dst_blocks_.T % 16)
                                                 ] = dst_chunk.block_palette.get_add_block(
-                                                    block
+                                                    transformed_block
                                                 )
 
                                                 src_blocks_ = src_blocks[mask]
@@ -249,6 +255,7 @@ def clone(
                     sum_progress += volumes[box_index]
 
         else:
+            # the selection can be cloned as is
             # the transform from the structure location to the world location
             offset = numpy.asarray(location).astype(int) - rotation_point
             moved_min_location = src_selection.min_array + offset
@@ -346,11 +353,26 @@ def clone(
                                     )
                                 )
 
-                    mask = paste_blocks[src_chunk.blocks[src_slices]]
-                    dst_chunk.blocks[dst_slices][mask] = lut[
-                        src_chunk.blocks[src_slices]
-                    ][mask]
-                    dst_chunk.changed = True
+                    try:
+                        block_mask = src_chunk.blocks[src_slices]
+                        mask = paste_blocks[block_mask]
+                        dst_chunk.blocks[dst_slices][mask] = lut[
+                            src_chunk.blocks[src_slices]
+                        ][mask]
+                        dst_chunk.changed = True
+                    except IndexError as e:
+                        locals_copy = locals().copy()
+                        import traceback
+
+                        numpy_threshold = numpy.get_printoptions()["threshold"]
+                        numpy.set_printoptions(threshold=sys.maxsize)
+                        with open("clone_error.log", "w") as f:
+                            for k, v in locals_copy.items():
+                                f.write(f"{k}: {v}\n\n")
+                        numpy.set_printoptions(threshold=numpy_threshold)
+                        raise IndexError(
+                            f"Error pasting.\nPlease notify the developers and include the clone_error.log file.\n{e}"
+                        ) from e
 
                 if include_entities:
                     # TODO: implement pasting entities when we support entities
