@@ -13,6 +13,8 @@ from typing import (
     Callable,
     Type,
     Union,
+    TypeVar,
+    Generic,
 )
 import copy
 import numpy
@@ -42,7 +44,6 @@ from amulet.api.data_types import (
     ChunkCoordinates,
     Dimension,
     PlatformType,
-    VersionIdentifierType,
 )
 from amulet.api.selection import SelectionGroup, SelectionBox
 from amulet.api.player import Player
@@ -50,24 +51,22 @@ from amulet.api.player import Player
 if TYPE_CHECKING:
     from amulet.api.wrapper.chunk.translator import Translator
 
-
-DefaultPlatform = "Unknown Platform"
-DefaultVersion = (0, 0, 0)
-
 DefaultSelection = SelectionGroup(
     SelectionBox((-30_000_000, 0, -30_000_000), (30_000_000, 256, 30_000_000))
 )
 
+VersionNumberT = TypeVar("VersionNumberT", int, Tuple[int, ...])
 
-class FormatWrapper(ABC):
+
+class FormatWrapper(Generic[VersionNumberT], ABC):
     """
     The FormatWrapper class is a class that sits between the serialised world or structure data and the program using amulet-core.
 
     It is used to access data from the serialised source in the universal format and write them back again.
     """
 
-    _platform: PlatformType
-    _version: VersionNumberAny
+    _platform: Optional[PlatformType]
+    _version: Optional[VersionNumberT]
 
     def __init__(self, path: str):
         """
@@ -81,8 +80,8 @@ class FormatWrapper(ABC):
         self._is_open = False
         self._has_lock = False
         self._translation_manager = None
-        self._platform: PlatformType = DefaultPlatform
-        self._version: VersionNumberAny = DefaultVersion
+        self._platform = None
+        self._version = None
         self._bounds: Dict[Dimension, SelectionGroup] = {}
         self._changed: bool = False
 
@@ -148,15 +147,23 @@ class FormatWrapper(ABC):
     @property
     def platform(self) -> PlatformType:
         """Platform string the data is stored in (eg "bedrock" / "java" / ...)"""
+        if self._platform is None:
+            raise Exception(
+                "Cannot access the game platform until the level has been loaded."
+            )
         return self._platform
 
     @property
-    def version(self) -> VersionNumberAny:
+    def version(self) -> VersionNumberT:
         """The version number for the given platform the data is stored in eg (1, 16, 2)"""
+        if self._version is None:
+            raise Exception(
+                "Cannot access the game version until the level has been loaded."
+            )
         return self._version
 
     @property
-    def max_world_version(self) -> VersionIdentifierType:
+    def max_world_version(self) -> Tuple[PlatformType, VersionNumberT]:
         """
         The version the world was last opened in.
 
@@ -237,7 +244,7 @@ class FormatWrapper(ABC):
 
     def _get_interface_and_translator(
         self, raw_chunk_data=None
-    ) -> Tuple[api_wrapper.Interface, "Translator", "VersionNumberAny"]:
+    ) -> Tuple[api_wrapper.Interface, "Translator", VersionNumberAny]:
         interface = self._get_interface(raw_chunk_data)
         translator, version_identifier = interface.get_translator(
             self.max_world_version, raw_chunk_data
@@ -435,7 +442,7 @@ class FormatWrapper(ABC):
             raise e
         except Exception as e:
             log.error(msg.format(*args), exc_info=True)
-            raise load_error(e)
+            raise load_error(e) from e
 
     def load_chunk(self, cx: int, cz: int, dimension: Dimension) -> Chunk:
         """
@@ -477,11 +484,11 @@ class FormatWrapper(ABC):
         )
 
         # decode the raw chunk data into the universal format
-        chunk, chunk_palette = self._decode(
+        chunk, block_palette = self._decode(
             interface, dimension, cx, cz, raw_chunk_data
         )
-        chunk_palette: AnyNDArray
-        chunk = self._unpack(translator, game_version, chunk, chunk_palette)
+        block_palette: AnyNDArray
+        chunk = self._unpack(translator, game_version, chunk, block_palette)
         return self._convert_to_load(
             chunk, translator, game_version, dimension, recurse=recurse
         )
@@ -501,10 +508,10 @@ class FormatWrapper(ABC):
         translator: "Translator",
         game_version: VersionNumberAny,
         chunk: Chunk,
-        chunk_palette: AnyNDArray,
+        block_palette: AnyNDArray,
     ) -> Chunk:
         return translator.unpack(
-            game_version, self.translation_manager, chunk, chunk_palette
+            game_version, self.translation_manager, chunk, block_palette
         )
 
     def _convert_to_load(
@@ -576,7 +583,7 @@ class FormatWrapper(ABC):
         chunk, chunk_palette = self._pack(chunk, translator, chunk_version)
         raw_chunk_data = self._encode(interface, chunk, dimension, chunk_palette)
 
-        self.put_raw_chunk_data(cx, cz, raw_chunk_data, dimension)
+        self._put_raw_chunk_data(cx, cz, raw_chunk_data, dimension)
 
     def _convert_to_save(
         self,
