@@ -15,7 +15,17 @@ from io import BytesIO
 import struct
 import copy
 
-import amulet_nbt
+from amulet_nbt import (
+    ByteTag,
+    IntTag,
+    StringTag,
+    ListTag,
+    CompoundTag,
+    ByteArrayTag,
+    IntArrayTag,
+    NamedTag,
+    load as load_nbt,
+)
 
 from amulet import log
 from amulet.api.data_types import (
@@ -30,7 +40,7 @@ from amulet.api.registry import BlockManager
 from amulet.api.wrapper import StructureFormatWrapper
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionGroup, SelectionBox
-from amulet.api.errors import ChunkDoesNotExist, ObjectWriteError, ObjectReadError
+from amulet.api.errors import ChunkDoesNotExist, ObjectWriteError
 
 from .section import ConstructionSection
 from .interface import Construction0Interface, ConstructionInterface
@@ -70,13 +80,16 @@ max_format_version = 0
 max_section_version = 0
 
 
-class ConstructionFormatWrapper(StructureFormatWrapper):
+class ConstructionFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
     """
     This FormatWrapper class exists to interface with the construction format.
     """
 
-    _platform: PlatformType
-    _version: VersionNumberTuple
+    _format_version: int
+    _section_version: int
+    _chunk_to_section: Dict[Tuple[int, int], List[ConstructionSection]]
+    _selection_boxes: List[SelectionBox]
+    _chunk_to_box: Dict[Tuple[int, int], List[SelectionBox]]
 
     def __init__(self, path: str):
         """
@@ -88,16 +101,16 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
         """
         super().__init__(path)
 
-        self._format_version: int = max_format_version
-        self._section_version: int = max_section_version
+        self._format_version = max_format_version
+        self._section_version = max_section_version
 
         # which sections are in a given chunk
-        self._chunk_to_section: Dict[Tuple[int, int], List[ConstructionSection]] = {}
+        self._chunk_to_section = {}
 
-        self._selection_boxes: List[SelectionBox] = []
+        self._selection_boxes = []
 
         # which selection boxes intersect a given chunk (boxes are clipped to the size of the chunk)
-        self._chunk_to_box: Dict[Tuple[int, int], List[SelectionBox]] = {}
+        self._chunk_to_box = {}
 
         self._shallow_load()
 
@@ -118,22 +131,24 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                             ]
                             f.seek(metadata_start)
 
-                            metadata = amulet_nbt.load(
+                            metadata = load_nbt(
                                 f.read(metadata_end - metadata_start),
                                 compressed=True,
-                            )
+                            ).compound
 
-                            self._platform = metadata["export_version"]["edition"].value
+                            export_version = metadata.get_compound("export_version")
+
+                            self._platform = export_version.get_string("edition").py_str
                             self._version = tuple(
                                 map(
-                                    lambda v: v.value,
-                                    metadata["export_version"]["version"],
+                                    lambda v: v.py_int,
+                                    export_version.get_list("version"),
                                 )
                             )
 
                             selection_boxes = (
-                                metadata["selection_boxes"]
-                                .value.reshape(-1, 6)
+                                metadata.get_int_array("selection_boxes")
+                                .np_array.reshape(-1, 6)
                                 .tolist()
                             )
                             self._bounds[self.dimensions[0]] = SelectionGroup(
@@ -193,24 +208,29 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
             metadata_start = INT_STRUCT.unpack(f.read(INT_STRUCT.size))[0]
             f.seek(metadata_start)
 
-            metadata = amulet_nbt.load(
+            metadata = load_nbt(
                 f.read(metadata_end - metadata_start),
                 compressed=True,
-            )
+            ).compound
 
             try:
-                self._platform = metadata["export_version"]["edition"].value
+                export_version = metadata.get_compound("export_version")
+                self._platform = export_version.get_string("edition").py_str
                 self._version = tuple(
-                    map(lambda v: v.value, metadata["export_version"]["version"])
+                    map(lambda v: v.py_int, export_version.get_list("version"))
                 )
             except KeyError as e:
                 raise KeyError(f'Missing export version identifying key "{e.args[0]}"')
 
-            self._section_version = metadata["section_version"].value
+            self._section_version = metadata.get_byte("section_version").py_int
 
-            palette = unpack_palette(metadata["block_palette"])
+            palette = unpack_palette(metadata.get_list("block_palette"))
 
-            selection_boxes = metadata["selection_boxes"].value.reshape(-1, 6).tolist()
+            selection_boxes = (
+                metadata.get_int_array("selection_boxes")
+                .np_array.reshape(-1, 6)
+                .tolist()
+            )
 
             self._bounds[self.dimensions[0]] = SelectionGroup(
                 [
@@ -222,7 +242,9 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
             self._populate_chunk_to_box()
 
             section_index_table = (
-                metadata["section_index_table"].value.view(SECTION_ENTRY_TYPE).tolist()
+                metadata.get_byte_array("section_index_table")
+                .np_array.view(SECTION_ENTRY_TYPE)
+                .tolist()
             )
 
             if self._section_version == 0:
@@ -237,15 +259,25 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                     length,
                 ) in section_index_table:
                     f.seek(position)
-                    nbt_obj = amulet_nbt.load(f.read(length))
-                    if nbt_obj["blocks_array_type"].value == -1:
+                    nbt_obj = load_nbt(f.read(length)).compound
+                    blocks_array_type = nbt_obj.get_byte("blocks_array_type").py_int
+                    if blocks_array_type == -1:
                         blocks = None
                         block_entities = None
                     else:
-                        blocks = numpy.reshape(
-                            nbt_obj["blocks"].value, (shape_x, shape_y, shape_z)
+                        if blocks_array_type == 7:
+                            block_array = nbt_obj.get_byte_array("blocks").np_array
+                        elif blocks_array_type == 11:
+                            block_array = nbt_obj.get_int_array("blocks").np_array
+                        elif blocks_array_type == 12:
+                            block_array = nbt_obj.get_long_array("blocks").np_array
+                        else:
+                            raise TypeError
+
+                        blocks = block_array.reshape((shape_x, shape_y, shape_z))
+                        block_entities = parse_block_entities(
+                            nbt_obj.get_list("block_entities")
                         )
-                        block_entities = parse_block_entities(nbt_obj["block_entities"])
 
                     start = numpy.array([start_x, start_y, start_z])
                     chunk_index: numpy.ndarray = start // self.sub_chunk_size
@@ -265,7 +297,7 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                             (shape_x, shape_y, shape_z),
                             blocks,
                             palette,
-                            parse_entities(nbt_obj["entities"]),
+                            parse_entities(nbt_obj.get_list("entities")),
                             block_entities,
                         )
                     )
@@ -317,13 +349,11 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
         f.write(magic_num)
         f.write(struct.pack(">B", self._format_version))
         if self._format_version == 0:
-            metadata = amulet_nbt.NBTFile(
-                amulet_nbt.TAG_Compound(
+            metadata = NamedTag(
+                CompoundTag(
                     {
-                        "created_with": amulet_nbt.TAG_String(
-                            "amulet_python_wrapper_v2"
-                        ),
-                        "selection_boxes": amulet_nbt.TAG_Int_Array(
+                        "created_with": StringTag("amulet_python_wrapper_v2"),
+                        "selection_boxes": IntArrayTag(
                             [
                                 c
                                 for box in self._bounds[
@@ -332,13 +362,11 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                                 for c in (*box.min, *box.max)
                             ]
                         ),
-                        "section_version": amulet_nbt.TAG_Byte(self._section_version),
-                        "export_version": amulet_nbt.TAG_Compound(
+                        "section_version": ByteTag(self._section_version),
+                        "export_version": CompoundTag(
                             {
-                                "edition": amulet_nbt.TAG_String(self._platform),
-                                "version": amulet_nbt.TAG_List(
-                                    [amulet_nbt.TAG_Int(v) for v in self._version]
-                                ),
+                                "edition": StringTag(self._platform),
+                                "version": ListTag([IntTag(v) for v in self._version]),
                             }
                         ),
                     }
@@ -358,12 +386,10 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                         section_palette = section.palette
                         position = f.tell()
 
-                        _tag = amulet_nbt.TAG_Compound(
-                            {"entities": serialise_entities(entities)}
-                        )
+                        _tag = CompoundTag({"entities": serialise_entities(entities)})
 
                         if blocks is None:
-                            _tag["blocks_array_type"] = amulet_nbt.TAG_Byte(-1)
+                            _tag["blocks_array_type"] = ByteTag(-1)
                         else:
                             flattened_array = blocks.ravel()
                             index, flattened_array = numpy.unique(
@@ -377,15 +403,13 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                             )
                             flattened_array = lut[flattened_array]
                             array_type = find_fitting_array_type(flattened_array)
-                            _tag["blocks_array_type"] = amulet_nbt.TAG_Byte(
-                                array_type().tag_id
-                            )
+                            _tag["blocks_array_type"] = ByteTag(array_type().tag_id)
                             _tag["blocks"] = array_type(flattened_array)
                             _tag["block_entities"] = serialise_block_entities(
                                 block_entities or []
                             )
 
-                        amulet_nbt.NBTFile(_tag).save_to(f)
+                        NamedTag(_tag).save_to(f)
 
                         length = f.tell() - position
                         section_index_table.append(
@@ -396,7 +420,7 @@ class ConstructionFormatWrapper(StructureFormatWrapper):
                     f"This wrapper doesn't support any section version higher than {max_section_version}"
                 )
             metadata_start = f.tell()
-            metadata["section_index_table"] = amulet_nbt.TAG_Byte_Array(
+            metadata["section_index_table"] = ByteArrayTag(
                 numpy.array(section_index_table, dtype=SECTION_ENTRY_TYPE).view(
                     numpy.int8
                 )
