@@ -2,18 +2,32 @@ from __future__ import annotations
 
 import os
 import struct
+import warnings
 from typing import Tuple, Dict, Union, Optional, List, BinaryIO, Iterable, Any
 from io import BytesIO
 import shutil
 import traceback
 import time
 
-import amulet_nbt as nbt
+from amulet_nbt import (
+    AbstractBaseTag,
+    load as load_nbt,
+    NamedTag,
+    CompoundTag,
+    StringTag,
+    ByteTag,
+    IntTag,
+    ListTag,
+    LongTag,
+    FloatTag,
+    utf8_escape_decoder,
+    utf8_escape_encoder,
+)
 from amulet.api.player import Player, LOCAL_PLAYER
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionBox, SelectionGroup
 
-from amulet.libs.leveldb import LevelDB
+from amulet.libs.leveldb import LevelDB, LevelDBException, LevelDBEncrypted
 from amulet.utils.format_utils import check_all_exist
 from amulet.api.data_types import (
     ChunkCoordinates,
@@ -22,58 +36,103 @@ from amulet.api.data_types import (
     Dimension,
     AnyNDArray,
 )
-from amulet.api.wrapper import WorldFormatWrapper, DefaultVersion
+from amulet.api.wrapper import WorldFormatWrapper
 from amulet.api.errors import ObjectWriteError, ObjectReadError, PlayerDoesNotExist
 
-from amulet.libs.leveldb import LevelDBException
-from amulet.level.interfaces.chunk.leveldb.leveldb_chunk_versions import (
+from .interface.chunk.leveldb_chunk_versions import (
     game_to_chunk_version,
 )
-from .dimension import LevelDBDimensionManager, OVERWORLD, THE_NETHER, THE_END
-from amulet.level.interfaces.chunk.leveldb.base_leveldb_interface import (
-    BaseLevelDBInterface,
+from .dimension import (
+    LevelDBDimensionManager,
+    ChunkData,
+    OVERWORLD,
+    THE_NETHER,
+    THE_END,
 )
+from .interface.chunk import BaseLevelDBInterface, get_interface
 
 InternalDimension = Optional[int]
 
 
-class BedrockLevelDAT(nbt.NBTFile):
-    def __init__(self, path: str = None):
-        self._path: Optional[str] = path
-        self._level_dat_version = 8
-        super().__init__(nbt.TAG_Compound(), "")
-        if path is not None and os.path.isfile(path):
-            self.load_from(path)
+class BedrockLevelDAT(NamedTag):
+    _path: str
+    _level_dat_version: int
 
-    def load_from(self, path: str):
-        with open(path, "rb") as f:
-            self._level_dat_version = struct.unpack("<i", f.read(4))[0]
-            if 4 <= self._level_dat_version <= 8:
-                data_length = struct.unpack("<i", f.read(4))[0]
-                root_tag = nbt.load(
-                    f.read(data_length), compressed=False, little_endian=True
+    def __init__(
+        self, tag=None, name: str = "", path: str = None, level_dat_version: int = None
+    ):
+        if isinstance(tag, str):
+            warnings.warn(
+                "You must use BedrockLevelDAT.from_file to load from a file.",
+                FutureWarning,
+            )
+            super().__init__()
+            self._path = path = tag
+            self._level_dat_version = 8
+            if os.path.isfile(path):
+                self.load_from(path)
+            return
+        else:
+            if not (isinstance(path, str) and isinstance(level_dat_version, int)):
+                raise TypeError(
+                    "path and level_dat_version must be specified when constructing a BedrockLevelDAT instance."
                 )
-                self.name = root_tag.name
-                self.value = root_tag.value
+            super().__init__(tag, name)
+            self._path = path
+            self._level_dat_version = level_dat_version
+
+    @classmethod
+    def from_file(cls, path: str):
+        level_dat_version, name, tag = cls._read_from(path)
+        return cls(tag, name, path, level_dat_version)
+
+    @property
+    def path(self) -> Optional[str]:
+        return self._path
+
+    @staticmethod
+    def _read_from(path: str) -> Tuple[int, str, AbstractBaseTag]:
+        with open(path, "rb") as f:
+            level_dat_version = struct.unpack("<i", f.read(4))[0]
+            if 4 <= level_dat_version <= 9:
+                data_length = struct.unpack("<i", f.read(4))[0]
+                root_tag = load_nbt(
+                    f.read(data_length),
+                    compressed=False,
+                    little_endian=True,
+                    string_decoder=utf8_escape_decoder,
+                )
+                name = root_tag.name
+                value = root_tag.tag
             else:
                 # TODO: handle other versions
                 raise ObjectReadError(
-                    f"Unsupported level.dat version {self._level_dat_version}"
+                    f"Unsupported level.dat version {level_dat_version}"
                 )
+        return level_dat_version, name, value
+
+    def load_from(self, path: str):
+        self._level_dat_version, self.name, self.tag = self._read_from(path)
+
+    def reload(self):
+        self.load_from(self.path)
 
     def save(self, path: str = None):
-        path = path or self._path
-        if not path:
-            raise ObjectWriteError("No path given.")
-        self.save_to(path)
+        self.save_to(path or self._path)
 
     def save_to(
         self,
         filename_or_buffer: Union[str, BinaryIO] = None,
+        *,
         compressed=False,
         little_endian=True,
+        string_encoder=utf8_escape_encoder,
     ) -> Optional[bytes]:
-        payload = super().save_to(compressed=compressed, little_endian=little_endian)
+        payload = super().save_to(
+            compressed=compressed,
+            little_endian=little_endian,
+            string_encoder=string_encoder,
+        )
         buffer = BytesIO()
         buffer.write(struct.pack("<ii", self._level_dat_version, len(payload)))
         buffer.write(payload)
@@ -86,13 +145,17 @@ class BedrockLevelDAT(nbt.NBTFile):
             filename_or_buffer.write(buffer.getvalue())
 
 
-class LevelDBFormat(WorldFormatWrapper):
+class LevelDBFormat(WorldFormatWrapper[VersionNumberTuple]):
     """
     This FormatWrapper class exists to interface with the Bedrock world format.
     """
 
-    _platform: PlatformType
-    _version: VersionNumberTuple
+    # The leveldb database. Access it through the public property `level_db`
+    _db: Optional[LevelDB]
+    # A class to manage dimension data. This is private
+    _dimension_manager: Optional[LevelDBDimensionManager]
+
+    _root_tag: BedrockLevelDAT
 
     def __init__(self, path: str):
         """
@@ -104,10 +167,9 @@ class LevelDBFormat(WorldFormatWrapper):
         """
         super().__init__(path)
         self._platform = "bedrock"
-        self._root_tag: BedrockLevelDAT = BedrockLevelDAT(
-            os.path.join(path, "level.dat")
-        )
-        self._level_manager: Optional[LevelDBDimensionManager] = None
+        self._root_tag = BedrockLevelDAT.from_file(os.path.join(path, "level.dat"))
+        self._db = None
+        self._dimension_manager = None
         self._shallow_load()
 
     def _shallow_load(self):
@@ -120,7 +182,7 @@ class LevelDBFormat(WorldFormatWrapper):
         """Load the level.dat file and check the image file"""
         if os.path.isfile(os.path.join(self.path, "world_icon.jpeg")):
             self._world_image_path = os.path.join(self.path, "world_icon.jpeg")
-        self.root_tag = BedrockLevelDAT(os.path.join(self.path, "level.dat"))
+        self.root_tag = BedrockLevelDAT.from_file(os.path.join(self.path, "level.dat"))
 
     @staticmethod
     def is_valid(path: str):
@@ -132,7 +194,7 @@ class LevelDBFormat(WorldFormatWrapper):
 
     @property
     def version(self) -> VersionNumberTuple:
-        if self._version == DefaultVersion:
+        if self._version is None:
             self._version = self._get_version()
         return self._version
 
@@ -145,7 +207,12 @@ class LevelDBFormat(WorldFormatWrapper):
         For this format wrapper it returns a tuple of 3/4 ints (the game version number)
         """
         try:
-            return tuple([t.value for t in self.root_tag["lastOpenedWithVersion"]])
+            return tuple(
+                [
+                    t.py_int
+                    for t in self.root_tag.compound.get_list("lastOpenedWithVersion")
+                ]
+            )
         except Exception:
             return 1, 2, 0
 
@@ -155,42 +222,40 @@ class LevelDBFormat(WorldFormatWrapper):
         return self._root_tag
 
     @root_tag.setter
-    def root_tag(self, root_tag: Union[nbt.NBTFile, nbt.TAG_Compound, BedrockLevelDAT]):
-        if isinstance(root_tag, nbt.TAG_Compound):
-            self._root_tag.value = root_tag
-        elif isinstance(root_tag, BedrockLevelDAT):
-            self._root_tag = root_tag
-        elif isinstance(root_tag, nbt.NBTFile):
+    def root_tag(self, root_tag: Union[NamedTag, CompoundTag, BedrockLevelDAT]):
+        if isinstance(root_tag, CompoundTag):
+            self._root_tag.tag = root_tag
+        elif isinstance(root_tag, NamedTag):
             self._root_tag.name = root_tag.name
-            self._root_tag.value = root_tag.value
+            self._root_tag.tag = root_tag.compound
         else:
             raise ValueError(
-                "root_tag must be a TAG_Compound, NBTFile or BedrockLevelDAT"
+                "root_tag must be a CompoundTag, NamedTag or BedrockLevelDAT"
             )
 
     @property
-    def level_name(self):
-        return str(self.root_tag.get("LevelName", ""))
+    def level_name(self) -> str:
+        return self.root_tag.compound.get_string("LevelName", StringTag()).py_str
 
     @level_name.setter
     def level_name(self, value: str):
-        self.root_tag["LevelName"] = nbt.TAG_String(value)
+        self.root_tag.compound["LevelName"] = StringTag(value)
 
     @property
     def last_played(self) -> int:
-        return int(self.root_tag.get("LastPlayed", 0))
+        return self.root_tag.compound.get_long("LastPlayed", LongTag()).py_int
 
     @property
     def game_version_string(self) -> str:
         try:
-            return f'Bedrock {".".join(str(v.value) for v in self.root_tag["lastOpenedWithVersion"].value)}'
+            return f'Bedrock {".".join(str(v.py_int) for v in self.root_tag.compound.get_list("lastOpenedWithVersion"))}'
         except Exception:
             return f"Bedrock Unknown Version"
 
     @property
     def dimensions(self) -> List["Dimension"]:
         self._verify_has_lock()
-        return self._level_manager.dimensions
+        return self._dimension_manager.dimensions
 
     # def register_dimension(
     #     self, dimension_internal: int, dimension_name: Optional["Dimension"] = None
@@ -202,11 +267,30 @@ class LevelDBFormat(WorldFormatWrapper):
     #     :param dimension_name: The name of the dimension shown to the user.
     #     :return:
     #     """
-    #     self._level_manager.register_dimension(dimension_internal, dimension_name)
+    #     self._dimension_manager.register_dimension(dimension_internal, dimension_name)
 
-    def _get_interface_key(
-        self, raw_chunk_data: Optional[Dict[bytes, bytes]] = None
-    ) -> Tuple[str, int]:
+    @property
+    def level_db(self) -> LevelDB:
+        """The raw leveldb database."""
+        if self._db is None:
+            raise Exception(
+                "The world is not open. The leveldb database cannot be accessed."
+            )
+        return self._db
+
+    @property
+    def _level_manager(self) -> LevelDBDimensionManager:
+        warnings.warn(
+            "_level_manager attribute is depreciated. If you want to access the raw leveldb database it can be accessed through the level_db property."
+        )
+        return self._dimension_manager
+
+    def _get_interface(
+        self, raw_chunk_data: Optional[Any] = None
+    ) -> BaseLevelDBInterface:
+        return get_interface(self._get_interface_key(raw_chunk_data))
+
+    def _get_interface_key(self, raw_chunk_data: Optional[ChunkData] = None) -> int:
         if raw_chunk_data:
             if b"," in raw_chunk_data:
                 chunk_version = raw_chunk_data[b","][0]
@@ -215,11 +299,11 @@ class LevelDBFormat(WorldFormatWrapper):
         else:
             chunk_version = game_to_chunk_version(
                 self.max_world_version[1],
-                self.root_tag.get("experiments", {})
-                .get("caves_and_cliffs", nbt.TAG_Byte())
-                .value,
+                self.root_tag.compound.get_compound("experiments", CompoundTag())
+                .get_byte("caves_and_cliffs", ByteTag())
+                .py_int,
             )
-        return self.platform, chunk_version  # TODO: work out a valid default
+        return chunk_version
 
     def _decode(
         self,
@@ -253,13 +337,16 @@ class LevelDBFormat(WorldFormatWrapper):
         except:
             pass
         try:
-            self._level_manager = LevelDBDimensionManager(self.path)
+            self._db = LevelDB(os.path.join(self.path, "db"))
+            self._dimension_manager = LevelDBDimensionManager(self)
             self._is_open = True
             self._has_lock = True
-            experiments = self.root_tag.get("experiments", {})
+            experiments = self.root_tag.compound.get_compound(
+                "experiments", CompoundTag()
+            )
             if (
-                experiments.get("caves_and_cliffs", nbt.TAG_Byte()).value
-                or experiments.get("caves_and_cliffs_internal", nbt.TAG_Byte()).value
+                experiments.get_byte("caves_and_cliffs", ByteTag()).py_int
+                or experiments.get_byte("caves_and_cliffs_internal", ByteTag()).py_int
                 or self.version >= (1, 18)
             ):
                 self._bounds[OVERWORLD] = SelectionGroup(
@@ -283,7 +370,7 @@ class LevelDBFormat(WorldFormatWrapper):
                     (-30_000_000, 0, -30_000_000), (30_000_000, 256, 30_000_000)
                 )
             )
-        except OSError as e:
+        except LevelDBEncrypted as e:
             self._is_open = self._has_lock = False
             raise LevelDBException(
                 "It looks like this world is from the marketplace.\nThese worlds are encrypted and cannot be edited."
@@ -325,17 +412,15 @@ class LevelDBFormat(WorldFormatWrapper):
         ).version_number
         self._version = version + (0,) * (5 - len(version))
 
-        self.root_tag = root = nbt.TAG_Compound()
-        root["StorageVersion"] = nbt.TAG_Int(8)
-        root["lastOpenedWithVersion"] = nbt.TAG_List(
-            [nbt.TAG_Int(i) for i in self._version]
-        )
-        root["Generator"] = nbt.TAG_Int(1)
-        root["LastPlayed"] = nbt.TAG_Long(int(time.time()))
-        root["LevelName"] = nbt.TAG_String("World Created By Amulet")
+        self.root_tag = root = CompoundTag()
+        root["StorageVersion"] = IntTag(8)
+        root["lastOpenedWithVersion"] = ListTag([IntTag(i) for i in self._version])
+        root["Generator"] = IntTag(1)
+        root["LastPlayed"] = LongTag(int(time.time()))
+        root["LevelName"] = StringTag("World Created By Amulet")
 
         os.makedirs(self.path, exist_ok=True)
-        self.root_tag.save(os.path.join(self.path, "level.dat"))
+        self.root_tag.save()
 
         db = LevelDB(os.path.join(self.path, "db"), True)
         db.close()
@@ -350,35 +435,37 @@ class LevelDBFormat(WorldFormatWrapper):
 
     def _save(self):
         os.makedirs(self.path, exist_ok=True)
-        self._level_manager.save()
-        self.root_tag.save(os.path.join(self.path, "level.dat"))
+        self.root_tag.save()
         with open(os.path.join(self.path, "levelname.txt"), "w") as f:
             f.write(self.level_name)
 
     def _close(self):
-        self._level_manager.close()
+        self._db.close()
+        self._db = None
+        self._dimension_manager = None
+        self._actor_counter = None
 
     def unload(self):
         pass
 
     def all_chunk_coords(self, dimension: "Dimension") -> Iterable[ChunkCoordinates]:
         self._verify_has_lock()
-        yield from self._level_manager.all_chunk_coords(dimension)
+        yield from self._dimension_manager.all_chunk_coords(dimension)
 
     def has_chunk(self, cx: int, cz: int, dimension: Dimension) -> bool:
-        return self._level_manager.has_chunk(cx, cz, dimension)
+        return self._dimension_manager.has_chunk(cx, cz, dimension)
 
     def _delete_chunk(self, cx: int, cz: int, dimension: "Dimension"):
-        self._level_manager.delete_chunk(cx, cz, dimension)
+        self._dimension_manager.delete_chunk(cx, cz, dimension)
 
     def _put_raw_chunk_data(
-        self, cx: int, cz: int, data: Dict[bytes, bytes], dimension: "Dimension"
+        self, cx: int, cz: int, data: ChunkData, dimension: "Dimension"
     ):
-        return self._level_manager.put_chunk_data(cx, cz, data, dimension)
+        return self._dimension_manager.put_chunk_data(cx, cz, data, dimension)
 
     def _get_raw_chunk_data(
         self, cx: int, cz: int, dimension: "Dimension"
-    ) -> Dict[bytes, bytes]:
+    ) -> ChunkData:
         """
         Return the raw data as loaded from disk.
 
@@ -387,7 +474,7 @@ class LevelDBFormat(WorldFormatWrapper):
         :param dimension: The dimension to load the data from.
         :return: The raw chunk data.
         """
-        return self._level_manager.get_chunk_data(cx, cz, dimension)
+        return self._dimension_manager.get_chunk_data(cx, cz, dimension)
 
     def all_player_ids(self) -> Iterable[str]:
         """
@@ -395,7 +482,7 @@ class LevelDBFormat(WorldFormatWrapper):
         """
         yield from (
             pid[7:].decode("utf-8")
-            for pid, _ in self._level_manager._db.iterate(b"player_", b"player_\xFF")
+            for pid, _ in self._db.iterate(b"player_", b"player_\xFF")
         )
         if self.has_player(LOCAL_PLAYER):
             yield LOCAL_PLAYER
@@ -403,7 +490,7 @@ class LevelDBFormat(WorldFormatWrapper):
     def has_player(self, player_id: str) -> bool:
         if player_id != LOCAL_PLAYER:
             player_id = f"player_{player_id}"
-        return player_id.encode("utf-8") in self._level_manager._db
+        return player_id.encode("utf-8") in self._db
 
     def _load_player(self, player_id: str) -> Player:
         """
@@ -414,23 +501,23 @@ class LevelDBFormat(WorldFormatWrapper):
         :param player_id: The desired player id
         :return: A Player instance
         """
-        player_nbt = self._get_raw_player_data(player_id)
+        player_nbt = self._get_raw_player_data(player_id).compound
         dimension = player_nbt["DimensionId"]
-        if isinstance(dimension, nbt.TAG_Int) and 0 <= dimension <= 2:
+        if isinstance(dimension, IntTag) and IntTag(0) <= dimension <= IntTag(2):
             dimension_str = {
                 0: OVERWORLD,
                 1: THE_NETHER,
                 2: THE_END,
-            }[dimension.value]
+            }[dimension.py_int]
         else:
             dimension_str = OVERWORLD
 
         # get the players position
         pos_data = player_nbt.get("Pos")
         if (
-            isinstance(pos_data, nbt.TAG_List)
+            isinstance(pos_data, ListTag)
             and len(pos_data) == 3
-            and pos_data.list_data_type == nbt.TAG_Float.tag_id
+            and pos_data.list_data_type == FloatTag.tag_id
         ):
             position = tuple(map(float, pos_data))
             position = tuple(
@@ -442,9 +529,9 @@ class LevelDBFormat(WorldFormatWrapper):
         # get the players rotation
         rot_data = player_nbt.get("Rotation")
         if (
-            isinstance(rot_data, nbt.TAG_List)
+            isinstance(rot_data, ListTag)
             and len(rot_data) == 2
-            and rot_data.list_data_type == nbt.TAG_Float.tag_id
+            and rot_data.list_data_type == FloatTag.tag_id
         ):
             rotation = tuple(map(float, rot_data))
             rotation = tuple(
@@ -460,13 +547,18 @@ class LevelDBFormat(WorldFormatWrapper):
             rotation,
         )
 
-    def _get_raw_player_data(self, player_id: str) -> nbt.NBTFile:
+    def _get_raw_player_data(self, player_id: str) -> NamedTag:
         if player_id == LOCAL_PLAYER:
             key = player_id.encode("utf-8")
         else:
             key = f"player_{player_id}".encode("utf-8")
         try:
-            data = self._level_manager._db.get(key)
+            data = self._db.get(key)
         except KeyError:
             raise PlayerDoesNotExist(f"Player {player_id} doesn't exist")
-        return nbt.load(data, compressed=False, little_endian=True)
+        return load_nbt(
+            data,
+            compressed=False,
+            little_endian=True,
+            string_decoder=utf8_escape_decoder,
+        )
