@@ -1,12 +1,18 @@
-from typing import TYPE_CHECKING, Any, Tuple, Union, Optional, List
+from typing import TYPE_CHECKING, Any, Tuple, Optional, List
 import numpy
 
-import amulet_nbt
+from amulet_nbt import IntTag, StringTag, CompoundTag
 
 from amulet.api.wrapper import Interface, EntityIDType, EntityCoordType
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionBox
-from amulet.api.data_types import AnyNDArray
+from amulet.api.data_types import (
+    AnyNDArray,
+    VersionNumberTuple,
+    VersionIdentifierType,
+    VersionNumberAny,
+    PlatformType,
+)
 from amulet.level.loader import Translators
 from amulet.api.block import Block
 from .chunk import MCStructureChunk
@@ -42,7 +48,7 @@ class MCStructureInterface(Interface):
                 Block(
                     namespace="minecraft",
                     base_name="air",
-                    properties={"block_data": amulet_nbt.TAG_Int(0)},
+                    properties={"block_data": IntTag(0)},
                 ),
             ),
         )
@@ -50,18 +56,18 @@ class MCStructureInterface(Interface):
         for index, blocks in enumerate(data.palette):
             block_layers: List[Tuple[Optional[int], Block]] = []
             for block in blocks:
-                namespace, base_name = block["name"].value.split(":", 1)
+                namespace, base_name = block["name"].py_str.split(":", 1)
                 if "version" in block:
-                    version: Optional[int] = block["version"].value
+                    version: Optional[int] = block["version"].py_int
                 else:
                     version = None
 
                 if "states" in block:  # 1.13 format
-                    properties = block["states"].value
+                    properties = block["states"].py_dict
                     if version is None:
                         version = 17694720  # 1, 14, 0, 0
                 else:
-                    properties = {"block_data": amulet_nbt.TAG_Int(block["val"].value)}
+                    properties = {"block_data": IntTag(block["val"].py_int)}
                 block_layers.append(
                     (
                         version,
@@ -96,7 +102,7 @@ class MCStructureInterface(Interface):
         self,
         chunk: "Chunk",
         palette: AnyNDArray,
-        max_world_version: Tuple[str, Union[int, Tuple[int, int, int]]],
+        max_world_version: VersionIdentifierType,
         box: SelectionBox,
     ) -> MCStructureChunk:
         """
@@ -114,7 +120,7 @@ class MCStructureInterface(Interface):
                 entities.append(
                     self._encode_entity(
                         e, self._entity_id_type, self._entity_coord_type
-                    ).value
+                    ).compound
                 )
         block_entities = []
         for e in chunk.block_entities:
@@ -122,7 +128,7 @@ class MCStructureInterface(Interface):
                 block_entities.append(
                     self._encode_block_entity(
                         e, self._block_entity_id_type, self._block_entity_coord_type
-                    ).value
+                    ).compound
                 )
 
         slices = box.create_moved_box(
@@ -133,16 +139,14 @@ class MCStructureInterface(Interface):
         for index, block_layers in enumerate(palette):
             blocks_out = []
             for version, block in block_layers:
-                block = amulet_nbt.TAG_Compound(
+                block = CompoundTag(
                     {
-                        "name": amulet_nbt.TAG_String(
-                            f"{block.namespace}:{block.base_name}"
-                        ),
-                        "states": amulet_nbt.TAG_Compound(block.properties),
+                        "name": StringTag(f"{block.namespace}:{block.base_name}"),
+                        "states": CompoundTag(block.properties),
                     }
                 )
                 if version:
-                    block["version"] = amulet_nbt.TAG_Int(version)
+                    block["version"] = IntTag(version)
                 blocks_out.append(block)
             out_palette[index] = blocks_out
 
@@ -156,10 +160,10 @@ class MCStructureInterface(Interface):
 
     def get_translator(
         self,
-        max_world_version: Tuple[str, Tuple[int, int, int]],
+        max_world_version: Tuple[PlatformType, VersionNumberTuple],
         data: Any = None,
         translation_manager: "TranslationManager" = None,
-    ) -> Tuple["Translator", Union[int, Tuple[int, int, int]]]:
+    ) -> Tuple["Translator", VersionNumberAny]:
         platform, version_number = max_world_version
         if platform != "bedrock":
             raise ValueError("Platform must be bedrock")
