@@ -1,4 +1,4 @@
-from typing import TYPE_CHECKING, Any, Tuple, Union
+from typing import TYPE_CHECKING, Any, Tuple
 import numpy
 
 from amulet.api.wrapper import Interface, EntityIDType, EntityCoordType
@@ -6,7 +6,13 @@ from .chunk import SchematicChunk
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionBox
 from amulet.level.loader import Translators
-from amulet.api.data_types import AnyNDArray
+from amulet.api.data_types import (
+    AnyNDArray,
+    VersionNumberAny,
+    VersionIdentifierType,
+    PlatformType,
+    VersionNumberTuple,
+)
 
 if TYPE_CHECKING:
     from amulet.api.wrapper import Translator
@@ -63,7 +69,7 @@ class SchematicInterface(Interface):
         self,
         chunk: "Chunk",
         palette: AnyNDArray,
-        max_world_version: Tuple[str, Union[int, Tuple[int, int, int]]],
+        max_world_version: VersionIdentifierType,
         box: SelectionBox,
     ) -> SchematicChunk:
         """
@@ -81,7 +87,7 @@ class SchematicInterface(Interface):
                 entities.append(
                     self._encode_entity(
                         e, self._entity_id_type, self._entity_coord_type
-                    ).value
+                    ).compound
                 )
         block_entities = []
         for e in chunk.block_entities:
@@ -89,7 +95,7 @@ class SchematicInterface(Interface):
                 block_entities.append(
                     self._encode_block_entity(
                         e, self._block_entity_id_type, self._block_entity_coord_type
-                    ).value
+                    ).compound
                 )
 
         slices = box.create_moved_box(
@@ -107,10 +113,10 @@ class SchematicInterface(Interface):
 
     def get_translator(
         self,
-        max_world_version: Tuple[str, Tuple[int, int, int]],
+        max_world_version: Tuple[PlatformType, VersionNumberTuple],
         data: Any = None,
         translation_manager: "TranslationManager" = None,
-    ) -> Tuple["Translator", Union[int, Tuple[int, int, int]]]:
+    ) -> Tuple["Translator", VersionNumberAny]:
         platform, version_number = max_world_version
         if platform == "java":
             version = translation_manager.get_version(platform, version_number)
