@@ -1,10 +1,10 @@
-from typing import TYPE_CHECKING, Any, Tuple, Union
+from typing import TYPE_CHECKING, Any, Tuple
 import numpy
 
 from amulet.api.wrapper import Interface, EntityIDType, EntityCoordType
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionBox
-from amulet.api.data_types import AnyNDArray
+from amulet.api.data_types import AnyNDArray, VersionIdentifierType, VersionNumberAny
 from amulet.level.loader import Translators
 from amulet.api.block import Block
 from .chunk import SpongeSchemChunk
@@ -59,7 +59,7 @@ class SpongeSchemInterface(Interface):
         self,
         chunk: "Chunk",
         palette: AnyNDArray,
-        max_world_version: Tuple[str, Union[int, Tuple[int, int, int]]],
+        max_world_version: VersionIdentifierType,
         box: SelectionBox,
     ) -> SpongeSchemChunk:
         """
@@ -77,7 +77,7 @@ class SpongeSchemInterface(Interface):
                 entities.append(
                     self._encode_entity(
                         e, self._entity_id_type, self._entity_coord_type
-                    ).value
+                    ).compound
                 )
         block_entities = []
         for e in chunk.block_entities:
@@ -85,7 +85,7 @@ class SpongeSchemInterface(Interface):
                 block_entities.append(
                     self._encode_block_entity(
                         e, self._block_entity_id_type, self._block_entity_coord_type
-                    ).value
+                    ).compound
                 )
 
         slices = box.create_moved_box(
@@ -105,7 +105,7 @@ class SpongeSchemInterface(Interface):
         max_world_version: Tuple[str, int],
         data: Any = None,
         translation_manager: "TranslationManager" = None,
-    ) -> Tuple["Translator", Union[int, Tuple[int, int, int]]]:
+    ) -> Tuple["Translator", VersionNumberAny]:
         platform, version_number = max_world_version
         if platform != "java":
             raise ValueError("Platform must be java")
