@@ -118,77 +118,173 @@ def to_nibble_array(arr: ndarray) -> ndarray:
     return (arr[::2] + (arr[1::2] << 4)).astype("uint8")
 
 
+"""
+Minecraft Java edition stores the block and height arrays in a compacted long array format.
+The format stores one or more entries per long, using the fewest number of bits required to store the data.
+There are two storage methods, the compact version was used prior to 1.16 and the less compact version in 1.16 and above.
+Apparently the less compact version is quicker to pack and unpack.
+The compact version effectively stores the values as a bit array spanning one or more values in the long array.
+There may be some padding if the bit array does not fill all the long values. (The letter "P" signifies an unused padding bit)
+PPAAAAAAAAABBBBBBBBBCCCCCCCCCDDDDDDDDDEEEEEEEEEFFFFFFFFFGGGGGGGG GHHHHHHHHHIIIIIIIIIJJJJJJJJJKKKKKKKKKLLLLLLLLLMMMMMMMMMOOOOOOOOO
+The less compact version does not allow entries to straddle long values. Instead, if required, there is padding within each long.
+PAAAAAAAAABBBBBBBBBCCCCCCCCCDDDDDDDDDEEEEEEEEEFFFFFFFFFGGGGGGGGG PHHHHHHHHHIIIIIIIIIJJJJJJJJJKKKKKKKKKLLLLLLLLLMMMMMMMMMOOOOOOOOO
+
+The functions below can be used to pack and unpack both formats.
+"""
+
+
 def decode_long_array(
-    long_array: numpy.ndarray, size: int, dense=True
+    long_array: numpy.ndarray,
+    size: int,
+    bits_per_entry: int,
+    dense=True,
+    signed: bool = False,
 ) -> numpy.ndarray:
     """
-    Decode an long array (from BlockStates or Heightmaps)
+    Decode a long array (from BlockStates or Heightmaps)
 
     :param long_array: Encoded long array
-    :param size: int: The expected size of the returned array
+    :param size: The expected size of the returned array
+    :param bits_per_entry: The number of bits per entry in the encoded array.
+    :param dense: If true the long arrays will be treated as a bit stream. If false they are distinct values with padding
+    :param signed: Should the returned array be signed.
     :return: Decoded array as numpy array
     """
+    # force the array to be a signed long array
     long_array = long_array.astype(">q")
-    bits_per_entry = (len(long_array) * 64) // size
+    # validate the inputs and throw an error if there is a problem
+    if isinstance(bits_per_entry, int):
+        assert (
+            1 <= bits_per_entry <= 64
+        ), f"bits_per_entry must be between 1 and 64 inclusive. Got {bits_per_entry}"
+        if dense:
+            expected_len = math.ceil(size * bits_per_entry / 64)
+        else:
+            expected_len = math.ceil(size / (64 // bits_per_entry))
+        if len(long_array) != expected_len:
+            raise Exception(
+                f"{'Dense e' if dense else 'E'}ncoded long array with {bits_per_entry} bits per entry should contain {expected_len} longs but got {len(long_array)}."
+            )
+    else:
+        raise ValueError(f"The bits_per_entry input must be an int.")
+    # unpack the long array into a bit array
     bits = numpy.unpackbits(long_array[::-1].astype(">i8").view("uint8"))
-    if not dense:
+    if dense:
+        if bits.size % bits_per_entry:
+            # if the array is densely packed and there is extra padding, remove it
+            bits = bits[bits.size % bits_per_entry :]
+    else:
+        # if not densely packed remove the padding per long
         entry_per_long = 64 // bits_per_entry
         bits = bits.reshape(-1, 64)[:, -entry_per_long * bits_per_entry :]
 
-    return numpy.packbits(
+    byte_length = 2 ** math.ceil(math.log(math.ceil(bits_per_entry / 8), 2))
+    dtype = {1: "B", 2: ">H", 4: ">I", 8: ">Q"}[byte_length]
+
+    # pad the bits to fill one of the above data types
+    arr = numpy.packbits(
         numpy.pad(
             bits.reshape(-1, bits_per_entry)[-size:, :],
-            [(0, 0), (16 - bits_per_entry, 0)],
+            [(0, 0), (byte_length * 8 - bits_per_entry, 0)],
             "constant",
         )
-    ).view(dtype=">h")[::-1]
+    ).view(dtype=dtype)[::-1]
+    if signed:
+        # convert to a signed array if requested
+        sarray = arr.astype({1: "b", 2: ">h", 4: ">i", 8: ">q"}[byte_length])
+        if bits_per_entry < 64:
+            mask = arr >= 2 ** (bits_per_entry - 1)
+            sarray[mask] = numpy.subtract(
+                arr[mask], 2**bits_per_entry, dtype=numpy.int64
+            )
+        arr = sarray
+    return arr
 
 
 def encode_long_array(
     array: numpy.ndarray,
-    dense: bool = True,
     bits_per_entry: Optional[int] = None,
-    min_bits_per_entry=4,
+    dense: bool = True,
+    min_bits_per_entry=1,
 ) -> numpy.ndarray:
     """
-    Encode an long array (from BlockStates or Heightmaps)
+    Encode a long array (from BlockStates or Heightmaps)
 
     :param array: A numpy array of the data to be encoded.
-    :param dense: If true the long arrays will be treated as a bit stream. If false they are distinct values with padding
     :param bits_per_entry: The number of bits to use to store each value. If left as None will use the smallest bits per entry.
+    :param dense: If true the long arrays will be treated as a bit stream. If false they are distinct values with padding
+    :param min_bits_per_entry: The mimimum value that bits_per_entry can be. If it is less than this it will be capped at this value.
     :return: Encoded array as numpy array
     """
-    array = array.astype(">h")
+    assert (
+        1 <= min_bits_per_entry <= 64
+    ), f"min_bits_per_entry must be between 1 and 64 inclusive. Got {bits_per_entry}"
+    # cast to a signed longlong array
+    array = array.astype(">q")
+    # work out how many bits are required to store the
     required_bits_per_entry = max(
-        int(numpy.amax(array)).bit_length(), min_bits_per_entry
+        max(
+            int(numpy.amin(array)).bit_length(),
+            int(numpy.amax(array)).bit_length(),
+        ),
+        min_bits_per_entry,
     )
     if bits_per_entry is None:
+        # if a bit depth has not been requested use the minimum required
         bits_per_entry = required_bits_per_entry
-    else:
+    elif isinstance(bits_per_entry, int):
+        assert (
+            1 <= bits_per_entry <= 64
+        ), f"bits_per_entry must be between 1 and 64 inclusive. Got {bits_per_entry}"
+        # if a bit depth has been set and it is smaller than what is required throw an error
         if required_bits_per_entry > bits_per_entry:
             raise Exception(
                 f"The array requires at least {required_bits_per_entry} bits per value which is more than the specified {bits_per_entry} bits"
             )
-    if not dense:
-        if bits_per_entry == 11:
-            bits_per_entry = 12  # 11 and 12 take up the same amount of space. I don't know if 11 exists any more.
-    bits = numpy.unpackbits(numpy.ascontiguousarray(array[::-1]).view("uint8")).reshape(
-        -1, 16
-    )[:, -bits_per_entry:]
-    if not dense:
+    else:
+        raise ValueError(
+            "bits_per_entry must be an int between 1 and 64 inclusive or None."
+        )
+    # make the negative values positive to make bit storage easier
+    uarray = array.astype(">Q")
+    if bits_per_entry < 64:
+        mask = array < 0
+        uarray[mask] = numpy.add(
+            array[mask], 2**bits_per_entry, dtype=numpy.uint64, casting="unsafe"
+        )
+    array = uarray
+
+    # unpack the individual values into a bit array
+    bits: numpy.ndarray = numpy.unpackbits(
+        numpy.ascontiguousarray(array[::-1]).view("uint8")
+    ).reshape(-1, 64)[:, -bits_per_entry:]
+    if dense:
+        if bits.size % 64:
+            # if the bit array does not fill a whole long
+            # add padding to the last long if required
+            bits = numpy.pad(
+                bits.ravel(),
+                [(64 - (bits.size % 64), 0)],
+                "constant",
+            )
+    else:
+        # if the array is not dense add padding
         entry_per_long = 64 // bits_per_entry
         if bits.shape[0] % entry_per_long:
+            # add padding to the last long if required
             bits = numpy.pad(
                 bits,
                 [(entry_per_long - (bits.shape[0] % entry_per_long), 0), (0, 0)],
                 "constant",
             )
+        # add padding for each long
         bits = numpy.pad(
             bits.reshape(-1, bits_per_entry * entry_per_long),
             [(0, 0), (64 - bits_per_entry * entry_per_long, 0)],
             "constant",
         )
 
+    # pack the bits into a long array
     return numpy.packbits(bits).view(dtype=">q")[::-1]
 
 
@@ -222,7 +318,7 @@ def get_smallest_dtype(arr: ndarray, uint: bool = True) -> int:
     :param uint: Should the array fit in uint or not (default: True)
     :return: The number of bits all the elements can be represented with
     """
-    possible_dtypes = (2 ** x for x in range(3, 8))
+    possible_dtypes = (2**x for x in range(3, 8))
     max_number = numpy.amax(arr)
     if not uint:
         max_number = max_number * 2
